<?php

/*******************************************************************************
       __        __
      / /   ____/ /___ _____     _________ ___
     / /   / __  / __ `/ __ \   / ___/ __ `__ \
    / /___/ /_/ / /_/ / /_/ /  / /__/ / / / / /
   /_____/\__,_/\__,_/ .___/   \___/_/ /_/ /_/
                    /_/  LDAP Contact Manager for Drupal

      Copyright (c) 2010 - Samuele ~redShadow~ Santi - Under GPL v3

********************************************************************************
  The entry editing form. Most part is taken from the CCK module node form,
  to implement cool stuff such as AHAH subforms.

  TODO Clean code and document all this..
  TODO find a way to implement caching of entries
  TODO do not fetch entries when not necessary
*******************************************************************************/


/**
 * @file
 * Manages the building of the multi-value entry editing form for
 * the LDAP Contact Manager module.
 * Most part of this code is derived from the cck/content module,
 * includes/content.node_form.inc
 */

/**
 * Entry editing form
 * Generates a form with support for multiple values via AHAH
 * @param array $form_state
 * @param int $dir_id
 * @param string $entry_dn
 * @return array
 */
function ldapwrite_entry_edit_form($form_state, $dir_id, $entry_dn = NULL) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');

  // get the schema for the current directory/entry
  $schema = ldapab_get_fields_schema($dir_id, $entry_dn);

  // get the directory object
  $directory = ldapab_get_directory($dir_id);

  if ($entry_dn) {
    // We are editing content
    drupal_set_title(t("Edit entry"));
    $entry = ldapab_get_entry($dir_id, $entry_dn);
  }
  else {
    // This is a new entry for the selected directory
    drupal_set_title(t("Add entry"));
    $entry = NULL;
  }

  // -- Create form ------------------------------------------------------------
  $form = array();
  $form['dir_id'] = array('#type' => 'value', '#value' => $dir_id);
  $form['entry_dn'] = array('#type' => 'value', '#value' => $entry_dn);

  // -- Add all the fields widgets to the form ---------------------------------
  foreach ($schema['fields'] as $field_name => $field) {
    $field['name'] = $field_name;

    // add informations to the #ldapab_field_info array
    $form['#ldapab_field_info'][$field_name] = $field;

    // merge the new widget(s) into the form
    $form += (array) ldapab_entry_field_form($form, $form_state, $field);
  }

  // -- Add the submit button --------------------------------------------------
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Save entry"),
    '#submit' => array("ldapwrite_entry_edit_form_submit"),
  );

  return $form;
}


/**
 * Create a separate form element for each field.
 *
 * @param $form
 *   the form to add this field element to
 * @param $form_state
 *   the form_state for the above form
 * @param $field
 *   the field array to use to create the form element
 * @param $get_delta
 *   use to get only a specific delta value of a multiple value field, otherwise
 *   function will return the entire $field form element.
 */
function ldapab_entry_field_form(&$form, &$form_state, $field, $get_delta = NULL) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $form['#cache'] = FALSE; // disable form caching

  // -- Load various values from form ------------------------------------------
  $field_name = $field['name'];
  $dir_id = $form['dir_id']['#value'];
  $entry_dn = $form['entry_dn']['#value'];

  // -- Load the selected entry ------------------------------------------------
  if ($dir_id && $entry_dn) {
    $entry = ldapab_get_entry($dir_id, $entry_dn);
  }
  else {
    $entry = NULL;
  }

  $addition = array(); // additions to be made to the form (array)
  $form_element = array(); // form element (container fieldset)
  $items = array(); // values to be filled in the widget (array of values)

  // -- Load values to $items --------------------------------------------------
  if (!empty($form_state['values'][$field_name])) {
    // Load from the form state
    $items = $form_state['values'][$field['name']];
    // If there was an AHAH add more button in this field, don't save it.
    unset($items[$field['name'] .'_add_more']);
  }
  elseif ($entry  && $entry->exists($field['name'])) {
    // Load values from the entry
    $items = $entry->getValue($field['name'], 'all');
  }

  // If content module handles multiple values for this form element,
  // and not selecting an individual $delta, process the multiple value form.
  if (!isset($get_delta)) {
    $form_element = ldapab_multiple_value_form($form, $form_state, $field, $items);
  }
  // TODO make sure the following lines are useless!
  // If the widget is handling multiple values (e.g optionwidgets),
  // or selecting an individual element, just get a single form
  // element and make it the $delta value.
//  else {
    //dpm("MARKER ldapwrite.editform.inc:122");
// ===== SUFF PENDING REMOVAL ==================================================
    //    $delta = isset($get_delta) ? $get_delta : 0;
    //    if ($element = $field['widget']) {
    //      //$title = check_plain($field['title']);
    //      $title = is_string($field['title']) ? check_plain($field['title']) : "";
    //      //$title = $field['title'];
    //      //$description = content_filter_xss(t($field['widget']['description']));
    //      $defaults = array(
    //        '#title' => $title,
    //        '#delta' => $delta,
    //        '#field_name' => $field['name'],
    //      );
    //      // If we're processing a specific delta value for a field where the
    //      // content module handles multiples, set the delta in the result.
    //      // For fields that handle their own processing, we can't make assumptions
    //      // about how the field is structured, just merge in the returned value.
    //      //if (content_handle('widget', 'multiple values', $field) == CONTENT_HANDLE_CORE) {
    //      $form_element[$delta] = array_merge($element, $defaults);
    //      //}
    //      //else {
    //      //  $form_element = array_merge($element, $defaults);
    //      //}
    //    }
// =============================================================================
//  }

  // Field name is needed at top level as well as the individual elements
  // so the multiple values or other field level theme or processing can find it.
  if ($form_element) {
    $defaults = array(
      '#tree' => TRUE,
      '#weight' => $field['widget']['#weight'],
      //'#count' => count($form_element),
    );
    $addition[$field['name']] = array_merge($form_element, $defaults);
  }

  return $addition;
}



/**
 * Special handling to create form elements for multiple values.
 *
 * Handles generic features for multiple fields:
 * - number of widgets
 * - AHAH-'add more' button
 * - drag-n-drop value reordering
 */
function ldapab_multiple_value_form(&$form, &$form_state, $field, $items) {
  $field_name = $field['name'];
  $dir_id = $form['dir_id']['#value'];
  $entry_dn = $form['entry_dn']['#value'];

  $entry = ldapab_get_entry($dir_id, $entry_dn);

  if ($items) {
    $values = $items;
  }
  elseif (isset($form_state['values'][$field_name])) {
    // TODO remove this if test never true
    if (LDAPWRITE_DEBUG) dpm("MARKER ldapab_multiple_value_form VALUES FROM FORM");
    // read values from form
    $values = $form_state['values'][$field_name];
  }
  elseif ($entry && $entry->exists($field_name)) {
    // TODO remove this if test never true
    if (LDAPWRITE_DEBUG) dpm("MARKER ldapab_multiple_value_form VALUES FROM LDAP");
    // values from ldap entry
    $values = $entry->getValue($field_name, 'all');
  }
  else {
    // empty array
    $values = array();
  }
  $values[] = "";

  //$title = check_plain($field['title']);
  $title = is_string($field['title']) ? check_plain($field['title']) : "untitled $field_name";

  // The fieldset container
  $form_element = array(
    //'#theme' => 'content_multiple_values',
    '#title' => $title,
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
  );

  for ($delta = 0; $delta < count($values); $delta++) {
    $element = $field['widget'];
    $defaults = array(
      //'#title' => "$title $delta",
      '#weight' => $delta,
      '#delta' => $delta,
      '#default_value' => $values[$delta],
      // Additional data
      '#field_name' => $field_name,
      '#dir_id' => $dir_id,
      '#entry_dn' => $entry_dn,
    );
    $form_element[$delta] = array_merge($element, $defaults);
  }

  // Add AHAH add more button, if not working with a programmed form.
  //if ($field['multiple'] == 1 && empty($form['#programmed'])) {
    // Make sure the form is cached so ahah can work.

  $form['#cache'] = TRUE;
  $field_name_css = str_replace('_', '-', $field_name);

  $form_element[$field_name .'_add_more'] = array(
    '#type' => 'submit',
    '#name' => $field_name .'_add_more',
    '#value' => t('Add Value'),
    '#weight' => count($values) + 1,
    '#submit' => array('ldapab_add_more_submit_proxy'),
    '#ahah' => array(
      'path' => 'ldapcm/js_add_more/'. $field_name .'/'. $dir_id .'/'. urlencode($entry_dn),
      'wrapper' => $field_name_css .'-items',
      'method' => 'replace',
      'effect' => 'fade',
    ),
    // When JS is disabled, the content_add_more_submit handler will find
    // the relevant field using these entries.
    '#field_name' => $field_name,
    '#dir_id' => $dir_id,
    '#entry_dn' => $entry_dn,
  );

  // Add wrappers for the fields and 'more' button.
  $form_element['#prefix'] = '<div id="'. $field_name_css .'-items">';
  $form_element['#suffix'] = '</div>';
  $form_element[$field_name .'_add_more']['#prefix'] = '<div class="content-add-more clear-block">';
  $form_element[$field_name .'_add_more']['#suffix'] =  '</div>';

  return $form_element;
}



/**
 * Submit handler to add more choices to a content form. This handler is used when
 * JavaScript is not available. It makes changes to the form state and the
 * entire form is rebuilt during the page reload.
 */
function ldapab_add_more_submit($form, &$form_state) {
  // Set the form to rebuild and run submit handlers.
  //node_form_submit_build_node($form, $form_state);
  $field_name = $form_state['clicked_button']['#field_name'];
  $dir_id = $form_state['clicked_button']['#dir_id'];
  $entry_dn = $form_state['clicked_button']['#entry_dn'];

  // Make the changes we want to the form state.
  if ($form_state['values'][$field_name][$field_name .'_add_more']) {
    $form_state['item_count'][$field_name] = count($form_state['values'][$field_name]);
  }
}




/**
 * Menu callback for AHAH addition of new empty widgets.
 */
function ldapab_add_more_js($field_name, $dir_id = NULL, $entry_dn = NULL) {
  //dpm("ldapab_add_more_js");
  module_load_include('inc', 'ldapab', 'ldapab.ldap');
  $schema = ldapab_get_fields_schema($dir_id, $entry_dn);
  $field = $schema['fields'][$field_name];

  if (!$field) {
    // Generate default field definition if not found in schema
    $field = array(
      'name' => $field_name,
      'widget' => array(
        '#type' => 'textfield',
        '#title' => $field_name,
      )
    );
  }
  $field['name'] = $field_name;
  //$field['#title'] = $field['title'];

  // Retrieve the cached form.
  $form_state = array('submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);
  if (!$form) {
    // Invalid form_build_id.
    drupal_json(array('data' => ''));
    exit;
  }

  // We don't simply return a new empty widget to append to existing ones, because
  // - ahah.js won't simply let us add a new row to a table
  // - attaching the 'draggable' behavior won't be easy
  // So we resort to rebuilding the whole table of widgets including the existing ones,
  // which makes us jump through a few hoops.

  // The form that we get from the cache is unbuilt. We need to build it so that
  // _value callbacks can be executed and $form_state['values'] populated.
  // We only want to affect $form_state['values'], not the $form itself
  // (built forms aren't supposed to enter the cache) nor the rest of $form_data,
  // so we use copies of $form and $form_data.

  $form_copy = $form;
  $form_state_copy = $form_state;
  $form_copy['#post'] = array();
  form_builder($_POST['form_id'], $form_copy, $form_state_copy);
  // Just grab the data we need.
  $form_state['values'] = $form_state_copy['values'];
  // Reset cached ids, so that they don't affect the actual form we output.
  form_clean_id(NULL, TRUE);

  // Sort the $form_state['values'] we just built *and* the incoming $_POST data
  // according to d-n-d reordering.
  unset($form_state['values'][$field_name][$field['name'] .'_add_more']);
//  foreach ($_POST[$field_name] as $delta => $item) {
//    $form_state['values'][$field_name][$delta]['_weight'] = $item['_weight'];
//  }
//  $form_state['values'][$field_name] = _content_sort_items($field, $form_state['values'][$field_name]);
//  $_POST[$field_name] = _content_sort_items($field, $_POST[$field_name]);

  // Build our new form element for the whole field, asking for one more element.
  $form_state['item_count'] = array($field_name => count($_POST[$field_name]) + 1);
  $form_element = ldapab_entry_field_form($form, $form_state, $field);
  // Let other modules alter it.
  drupal_alter('form', $form_element, array(), 'ldapab_add_more_js');

  // Add the new element at the right place in the (original, unbuilt) form.
  $form[$field_name] = $form_element[$field_name];

  // Save the new definition of the form.
  $form_state['values'] = array();
  form_set_cache($form_build_id, $form, $form_state);

  // Build the new form against the incoming $_POST values so that we can
  // render the new element.
  $delta = max(array_keys($_POST[$field_name])) + 1;
  //$_POST[$field_name][$delta]['_weight'] = $delta;
  $form_state = array('submitted' => FALSE);
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
  );
  $form = form_builder($_POST['form_id'], $form, $form_state);

  // Render the new output.
  $field_form = $form[$field_name];
  // We add a div around the new content to receive the ahah effect.
  $field_form[$delta]['#prefix'] = '<div class="ahah-new-content">'. (isset($field_form[$delta]['#prefix']) ? $field_form[$delta]['#prefix'] : '');
  $field_form[$delta]['#suffix'] = (isset($field_form[$delta]['#suffix']) ? $field_form[$delta]['#suffix'] : '') .'</div>';
  // Prevent duplicate wrapper.
  unset($field_form['#prefix'], $field_form['#suffix']);

  // If a newly inserted widget contains AHAH behaviors, they normally won't
  // work because AHAH doesn't know about those - it just attaches to the exact
  // form elements that were initially specified in the Drupal.settings object.
  // The new ones didn't exist then, so we need to update Drupal.settings
  // by ourselves in order to let AHAH know about those new form elements.
  $javascript = drupal_add_js(NULL, NULL);
  $output_js = isset($javascript['setting']) ? '<script type="text/javascript">jQuery.extend(Drupal.settings, '. drupal_to_js(call_user_func_array('array_merge_recursive', $javascript['setting'])) .');</script>' : '';

  //$field_form['#title'] = "$field_name";
  $field_form['#type'] = "fieldset";
  //$field_form['#value'] = dpm($field_form, TRUE);

  //dpm($field_form);

  $output = theme('status_messages') . drupal_render($field_form) . $output_js;

  // Using drupal_json() breaks filefield's file upload, because the jQuery
  // Form plugin handles file uploads in a way that is not compatible with
  // 'text/javascript' response type.
  $GLOBALS['devel_shutdown'] =  FALSE;
  //dpm("ldapab_add_more_js return");
  //dpm($output);
  print drupal_to_js(array('status' => TRUE, 'data' => $output));
  exit;
}


/**
 * Entry editing form submit callback
 * @param unknown_type $form
 * @param unknown_type $form_state
 * @return unknown_type
 * TODO add support for adding new entries (not only updating)
 */
function ldapwrite_entry_edit_form_submit($form, &$form_state) {
  module_load_include('inc', 'ldapab', 'ldapab.ldap');

  if (LDAPWRITE_DEBUG) {
    dpm("WOAH! The form was finally submitted..");
    dpm("\$form");
    dpm($form);
    dpm("\$form_state");
    dpm($form_state);
    dpm("Now, do something with the submitted values.. :)");
  }
  
  // Create the new entries to be replaced into the entry
  $dir_id = $form_state['values']['dir_id'];
  $entry_dn = $form_state['values']['entry_dn'];
  
  if (LDAPWRITE_DEBUG) {
    dpm("DIRECTORY $dir_id");
    dpm("ENTRY_DN $entry_dn");
  }
  
  $entry = ldapab_get_entry($dir_id, $entry_dn);
  if (!$entry) {
    drupal_set_message(t("Cannot fetch entry - unable to edit"), 'error');
    return;
  }
  
  // Build the list of attributes that could be edited, and load their values
  $schema = ldapab_get_fields_schema($dir_id, $entry_dn);
  $new_values = array();
  
  if (LDAPWRITE_DEBUG) {
    dpm("Original fields");
    dpm($schema['fields']);
  }
  
  // TODO store fields being edited in the form as a value somewhere?
  
  // For each field to be checked -> edited
  foreach ($schema['fields'] as $field_name => $field) {
    if (isset($form_state['values'][$field_name])) {
      // New values are passed
      $field_values = $form_state['values'][$field_name];
      unset($field_values[$field_name .'_add_more']);
      
      $new_values[$field_name] = array();
      
      foreach ($field_values as $key => $value) {
        if (is_numeric($key) && $value) {
          $new_values[$field_name][] = $value;
        }
      }
      // Check if values are set before updating!
      if ($new_values[$field_name]) {
        //if ($entry->exists($field_name)) {
          // Replace entries
          $entry->replace(array($field_name => $new_values[$field_name]));
        //}
        //else {
        //  $entry->add(array($field_name => $new_values[$field_name]));
        //}
      }
    }
  }
  
  if (LDAPWRITE_DEBUG) {
    dpm("New values to be replaced");
    dpm($new_values);
  }
  
  // Update the entry
  // TODO we colud replace() here the $new_values.. test it!
  $entry->update();
  
  if (!ldapab_is_error($result)) {
    drupal_set_message(t("Entry %name successfully saved!", array("%name" => $entry->getValue('cn', 'single'))));
  }
}


 