<?php

/*******************************************************************************
       __        __
      / /   ____/ /___ _____     _________ ___
     / /   / __  / __ `/ __ \   / ___/ __ `__ \
    / /___/ /_/ / /_/ / /_/ /  / /__/ / / / / /
   /_____/\__,_/\__,_/ .___/   \___/_/ /_/ /_/
                    /_/  LDAP Contact Manager for Drupal

  Copyright (c) 2010 - Samuele ~redShadow~ Santi - Under GPL v3
  This project is sponsored by 2V S.r.l. - http://www.squadrainformatica.com

********************************************************************************

  LDAP querying functions

  The PEAR classes should be called ONLY inside this file, to provide flexible
  APIs to connect to the LDAP directory.
  
  TODO find a way to translate PEAR error messages, extracting parameters..

*******************************************************************************/

// When this script is included, we need PEAR and Net_LDAP2
include_once "PEAR.php";
include_once "Net/LDAP2.php";

/**
 * Get the list of defined directories, from the database table
 * TODO move somewhere else out of here
 * @return array
 */
function ldapab_get_all_directories() {
  $res = db_query("SELECT * FROM {ldapab_directories}");
  $rows = array();
  while ($row = db_fetch_object($res)) {
    $row->connection = unserialize($row->connection);
    $row->options = unserialize($row->options);
    $row->searchbase = $row->options['searchbase'];
    $rows[$row->id] = $row;
  }
  return $rows;
}

/**
 * Gets the connection parameters to a given directory
 * TODO make this query the database, once it is defined..
 * TODO move somewhere else out of here
 * @param int $dir_id
 * @return object|false
 */
function ldapab_get_directory($dir_id) {
  $row = db_fetch_object(db_query_range("SELECT * FROM {ldapab_directories} WHERE id=%d", $dir_id, 0, 1));
  if ($row) {
    $row->connection = unserialize($row->connection);
    $row->options = unserialize($row->options);
    $row->searchbase = $row->options['searchbase'];
    return $row;
  }
  else {
    return FALSE;
  }
}

/**
 * Get LDAP connection for a given directory
 * @return mixed
 */
function ldapab_get_connection($dir_id) {
//  include_once "PEAR.php";
//  include_once "Net/LDAP2.php";
  // Connect to LDAP and fetch all the entries
  $config = ldapab_get_directory($dir_id);
  // Connecting using the configuration:
  $ldap = Net_LDAP2::connect($config->connection);
  if (PEAR::isError($ldap)) {
    drupal_set_message(t("Could not connect to LDAP-server: @message", array('@message' => $ldap->getMessage())), 'error');
    return FALSE;
  }
  return $ldap;
}

/**
 * List the available filter types in a name => description array
 * @return array
 */
function ldapab_get_filter_types() {
  return array(
    '' => "---",
    'equals' => t("Equals"),
    'begins' => t("Begins with"),
    'ends' => t("Ends with"),
    'contains' => t("Contains"),
    'present' => t("Is present"),
    'any' => t("Any value"),
    'greater' => t("Greater than"),
    'less' => t("Less than"),
    'greaterOrEqual' => t("Grater or equal"),
    'lessOrEqual' => t("Less or equal"),
    'approx' => t("Similar to"),
  );
}

/**
 * Get the list of fields to be included in the search form
 * @return unknown_type
 */
function ldapab_get_searchable_fields() {
  // TODO query the schemas to get available fields
  // TODO improve this by allowing custom-defined fields
  return array(
    // -- standard inetOrgPerson fields ----------------------------------------
    //'dn' => t("Distinguished Name"),
    //'cn' => t("Common Name"),
    //'displayName' => t("Display Name"),
    'givenName' => t("Name"),
    'sn' => t("Surname"),
    'o' => t("Organization"),
    'ou' => t("Organization Unit"),
    'mail' => t("E-Mail"),
    'homePhone' => t("Home Phone"),
    'mobile' => t("Mobile Phone"),
    //'objectClass' => t("Object Class"),
    //'userPassword' => t("Password"),
    //'postalAddress' => t("Address"),
    //'postalCode' => t("Postal Code"),
    'l' => t("Location"),
    // -- fields from juridicPersonIt schema by 2V S.r.l. ----------------------
    //'CodiceFiscale' => t("Codice Fiscale"),
    //'PartitaIva' => t("Partita IVA"),
    // -------------------------------------------------------------------------
  );
}


/**
 * Gets the field schema for a given directory.
 * TODO autogenerate this from the LDAP schema of the object
 * @param int $dir_id the directory id (for per-directory settings)
 * @param string $entry_dn the entry dn (to read custom objectClass)
 * @return array
 */
function ldapab_get_fields_schema($dir_id = NULL, $entry_dn = NULL) {
  $schema = array();
  $schema['fields'] = array();
  $schema['fields']['givenName'] = array(
    'title' => t("Name"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => TRUE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['sn'] = array(
    'title' => t("Surname"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => TRUE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['o'] = array(
    'title' => t("Organization"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => TRUE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['ou'] = array(
    'title' => t("Organizational Unit"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => TRUE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['mail'] = array(
    'title' => t("E-Mail"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => TRUE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['homePhone'] = array(
    'title' => t("Phone"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => FALSE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['mobile'] = array(
    'title' => t("Mobile phone"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => FALSE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  $schema['fields']['l'] = array(
    'title' => t("Location"),
    'widget' => array('#type' => 'textfield'),
    'allow search' => TRUE,
    'fulltext search' => FALSE,
    'search filters' => array('equals', 'begins', 'ends', 'contains', 'present', 'any', 'approx'),
  );
  return $schema;
}


/**
 * Parse a filter string
 * @param unknown_type $filter
 * @return unknown_type
 */
function ldapab_parse_filter_string($filter) {
  include_once "PEAR.php";
  include_once "Net/LDAP2.php";
  $result = Net_LDAP2_Filter::parse($filter);
  if (PEAR::isError($result)) {
    drupal_set_message(t("Could not connect to LDAP-server: @message", array('@message' => $ldap->getMessage())), 'error');
    return FALSE;
  }
  else {
    return $result;
  }
}

/**
 * Gets an entry object directly by DN
 * @param int $dir_id
 * @param string $entry_dn
 * @return object
 */
function ldapab_get_entry($dir_id, $entry_dn) {
  $ldap = ldapab_get_connection($dir_id);

  if (!$ldap || PEAR::isError($ldap)) {
    drupal_set_message(t("Cannot connect to LDAP"), 'error');
    return FALSE;
  }
  //dpm("Fetching entry: DIR=". $dir_id ." DN=". $entry_dn);
  $entry = $ldap->getEntry($entry_dn);
  //dpm($entry->getValues());
  if (PEAR::isError($entry)) {
    drupal_set_message($entry->getMessage(), 'error');
    return FALSE;
  }
  else {
    return $entry;
  }
}

/**
 * Gets the schema for a given entry
 * @param int $dir_id The entry DN
 * @param string $dn The (schema?) DN
 * @return object
 */
function ldapab_get_directory_schema($dir_id, $dn = NULL) {
  $ldap = ldapab_get_connection($dir_id);
  
  if (!ldapab_error_or_return($ldap)) {
    drupal_set_message(t("Cannot connect to LDAP"), 'error');
    return FALSE;
  }

  if ($dn) {
    $schema = $ldap->schema($dn);
  }
  else {
    $schema = $ldap->schema();
  }

  return ldapab_error_or_return($schema);
}


/**
 * Perform a search on the directory
 * @param unknown_type $dir_id
 * @param unknown_type $filter
 * @return unknown_type
 */
function ldapab_search_entries($dir_id, $filter) {
  $ldap = ldapab_get_connection($dir_id);
  $dir = ldapab_get_directory($dir_id);
  $searchbase = $dir->options['searchbase'];
  $options = array('scope' => 'sub');
  $search = $ldap->search($searchbase, $filter, $options);
  if (PEAR::isError($search)) {
    drupal_set_message($search->getMessage(), 'error');
    return FALSE;
  }
  else {
    return $search;
  }
}

/**
 * Create a full-text search filter
 * @param int $dir_id
 * @param string $text the text to search
 * @param array $filter
 * @return Net_LDAP2_Search object if success, or FALSE if failed.
 */
function ldapab_create_filter_fulltext($dir_id, $text, $fields = NULL) {
  if (!$fields) {
    $conf = ldapab_get_directory($dir_id);
    $fields = explode(",", $conf->options['default_fulltext_fields']);
  }
  $filters = array();
  foreach ($fields as $field) {
    $filters[] = array(
      'attr_name' => $field,
      'match' => 'contains',
      'value' => $text,
    );
  }
  $filter = ldabap_create_filter_from_form($filters, 'or');
  return $filter;
}

/**
 * Create filters from an array similar to the one created
 * by the search form submit handler
 * @param array $form_filters an array of filters from form values
 * @param string $operator the operator used to combine the filters
 * @return
 *  Net_LDAP2_Filter leaf/group object if filter creation was successful
 *  NULL if no filter where specified
 *  FALSE if error occurred while creating filters
 */
function ldabap_create_filter_from_form($form_filters, $operator = 'and') {
  include_once "PEAR.php";
  include_once "Net/LDAP2.php";
  // Array to contain all the Net_LDAP2_Filter classes
  $filters = array();
  foreach ($form_filters as $filter_def) {
    // If the 'match' operator is selected, process the filter
    if ($filter_def['match']) {
      $newfilter = Net_LDAP2_Filter::create($filter_def['attr_name'], $filter_def['match'], $filter_def['value']);
      
      if (ldapab_error_or_return($newfilter)) {
        $filters[] = $newfilter;
      }
      
//      if (PEAR::isError($newfilter)) {
//        // Error.. skip the filter and set a message
//        drupal_set_message($newfilter->getMessage(), 'error');
//      }
//      else {
//        // Ok. Append the filter to the list
//        $filters[] = $newfilter;
//      }
      
      
    }
  }

  if (count($filters) > 1) {
    // We have some filters to combine
    $filter = Net_LDAP2_Filter::combine($operator, $filters);
  }
  elseif (count($filters) == 1) {
    // Just one filter.. directly use
    $filter = $filters[0];
  }
  else {
    // No filters specified.. return NULL
    return NULL;
  }

  return ldapab_error_or_return($filter);
}

/**
 * Check if it is an error object
 * TODO is this really needed?
 * @param $object the object to check
 * @param $return whether to return the message or print it
 * to error messages.
 * @return mixed
 */
function ldapab_is_error($object, $return = FALSE) {
  include_once "PEAR.php";
  include_once "Net/LDAP2.php";
  if (PEAR::isError($object)) {
    if ($return) {
      // Return the message
      return $ldap->getMessage();
    }
    else {
      // Set error message
      drupal_set_message($ldap->getMessage(), 'error');
    }
    return TRUE; // yes, this was an error
  }
  else {
    return FALSE; // no, it isn't an error
  }
}

/**
 * If the object is an error, print the error message and return FALSE,
 * else return the passed $object itself
 * @param $object an object to check
 * @return mixed
 */
function ldapab_error_or_return($object) {
  include_once "PEAR.php";
  include_once "Net/LDAP2.php";
  
  if (PEAR::isError($object)) {
    drupal_set_message($ldap->getMessage(), 'error');
    return FALSE;
  }
  else {
    return $object;
  }
}
